import fs from "node:fs/promises";
import path from "node:path";
import stream from "node:stream";
import { describe, it, expect } from "vitest";
import YAML from "yaml";
import {
  createTestTmpdir,
  createSDK,
  keys,
  createInteractiveCLIHelpers,
} from "./utils.js";

const includes = (query: string) => (text: string) => text.includes(query);

describe("create-sdk CLI", { retry: 3, timeout: 175_000 }, () => {
  it("smoke test", async (t) => {
    const [tmpdir, cleanup] = await createTestTmpdir();
    t.onTestFinished(async () => {
      await cleanup();
    });

    const proc = createSDK({ all: true });
    t.onTestFinished(async () => {
      // If the process successfully started
      if (proc !== null) {
        // If the process is still running
        if (proc.exitCode === null) {
          await Promise.all([
            // close it...
            proc.kill("SIGKILL"),
            // ...and let it close
            proc,
          ]);
        }
      }
    });
    expect(proc.all).toBeInstanceOf(stream.Readable); // Sanity check
    expect(proc.stdin).toBeInstanceOf(stream.Writable); // Sanity check

    const output: string[] = [];
    proc.all!.on("data", (chunk: unknown) => {
      if (chunk instanceof Buffer || typeof chunk === "string") {
        output.push(chunk.toString());
      }
    });
    t.onTestFailed(() => {
      console.debug("Command exit code:", proc.exitCode);
      console.debug("Command output:", proc.exitCode);
      console.debug("===================");
      console.debug(output.length === 0 ? "<None>" : output.join("\n"));
    });

    const { enter, type, waitFor } = createInteractiveCLIHelpers({
      stdin: proc.stdin!,
      stdout: proc.all!,
    });

    await waitFor(
      includes("? Step 1/6: Where do you want to create the extension?"),
      { timeout: 45_000 },
    );
    await type(tmpdir);
    await enter();

    await waitFor(includes("? Step 2/6: What is the slug of the extension?"));
    await type("test-extension");
    await enter();

    await waitFor(
      includes("? Step 3/6: Which boilerplate should be included?"),
    );
    await type(keys.down);
    await waitFor(includes("❯ Connector"));
    await enter();

    await waitFor(includes("? - Will your connector support Netlify Connect?"));
    await enter();

    await waitFor(
      includes("? - Will your connector support Netlify Visual Editor?"),
    );
    await type(keys.down);
    await waitFor(includes("❯ No"));
    await enter();

    await waitFor(
      includes("? Step 4/6: Which package manager do you want to use?"),
    );
    await type(keys.down);
    await type(keys.down);
    await waitFor(includes("❯ npm"));
    await enter();

    await waitFor(includes("Step 5/6: Adding dependencies..."));

    // We're waiting for packages to finish installing here, which will take a while, so we'll
    // devote most of the test's timeout budget to this step.
    await waitFor(includes("Step 6/6: Initialising extension..."), {
      timeout: 120_000,
    });

    await Promise.all([
      waitFor(includes("Your extension was created! Time to build! 🚀")),
      waitFor(includes(`Enter your extension directory using cd ${tmpdir}`)),
      waitFor(includes("Done! ✅")),
    ]);

    const stats = await fs.stat(path.resolve(tmpdir, "extension.yaml"));
    expect(stats.isFile()).toBe(true);
    const config = YAML.parse(
      await fs.readFile(path.resolve(tmpdir, "extension.yaml"), "utf8"),
    );
    expect(config).toHaveProperty("config.scopes", {
      site: ["read", "write"],
      user: ["read", "write"],
      env: ["read", "write"],
    });
  });
});
