import stream from "node:stream";
import fs from "node:fs/promises";
import os from "node:os";
import path from "node:path";
import { type Options as ExecaOptions, execaNode } from "execa";
import { getBinPathSync } from "get-bin-path";
import stripAnsi from "strip-ansi";
import { promiseWithResolvers } from "../src/lib/promise_with_resolvers_polyfill.js";

export const createTestTmpdir = async (): Promise<
  [tmpdir: string, () => Promise<void>]
> => {
  const tmpdir = await fs.mkdtemp(path.join(os.tmpdir(), "create-sdk-test-"));
  return [
    tmpdir,
    async () => {
      await fs.rm(tmpdir, { recursive: true, force: true });
    },
  ];
};

/**
 * getCreateSDKBinEntrypointPath locates the (uncompiled) create-sdk binary entrypoint.
 */
const getCreateSDKBinEntrypointPath = (): string => {
  const builtBinPath = getBinPathSync();
  if (builtBinPath === undefined) {
    throw new Error("Failed to resolve CLI entrypoint");
  }
  const builtBinPathParts = builtBinPath.split(path.sep);
  const lastDistIndex = builtBinPathParts.lastIndexOf("dist");
  if (lastDistIndex === -1) {
    throw new Error("Failed to resolve CLI entrypoint");
  }
  const binPathParts = builtBinPathParts.slice();
  binPathParts[lastDistIndex] = "src";
  return binPathParts.join(path.sep);
};

/**
 * createSDK launches the create-sdk binary. It runs the uncompiled binary and so does not require
 * the project to be built first.
 *
 * Returns an execa Subprocess object.
 */
export const createSDK = (options: ExecaOptions = {}) =>
  execaNode(getCreateSDKBinEntrypointPath(), {
    reject: false,
    ...options,
    env: {
      GIT_AUTHOR_NAME: "Test User",
      GIT_AUTHOR_EMAIL: "test@test.com",
      GIT_COMMITTER_NAME: "Test User",
      GIT_COMMITTER_EMAIL: "test@test.com",
    },
    nodeOptions: ["--import", "tsx", ...(options?.nodeOptions ?? [])],
  });

export const findInStream = async <T = unknown>(
  stream: stream.Readable,
  fn: (chunk: unknown) => T | undefined,
  { timeout = 5000 }: { timeout?: number } = {},
): Promise<T> => {
  // We only ever reject from an event, which will have a useless stack attached to it. Use this so
  // we at least get some stacktrace that points to calling code
  const timeoutErr = new Error("timed out");
  const signal = AbortSignal.timeout(timeout);
  let hasPromiseSettled = false;
  const { promise, resolve, reject } = promiseWithResolvers<T>();

  const onMessage = (chunk: any) => {
    const result = fn(chunk);
    if (result !== undefined) {
      resolve(result);
      hasPromiseSettled = true;
      stream.off("data", onMessage);
    }
  };
  stream.on("data", onMessage);

  signal.addEventListener(
    "abort",
    () => {
      if (!hasPromiseSettled) {
        stream.off("data", onMessage);
        reject(timeoutErr);
        hasPromiseSettled = true;
      }
    },
    { once: true },
  );

  return promise;
};

export const waitForInStream = async (
  stream: stream.Readable,
  test: (chunk: unknown) => boolean,
  options: { timeout?: number } = {},
): Promise<void> => {
  await findInStream(stream, (chunk) => test(chunk) || undefined, options);
};

export const enter = (stream: stream.Writable) =>
  new Promise<void>((resolve, reject) => {
    stream.write(keys.enter, (err) => {
      if (err != null) {
        return reject(err);
      }
      resolve();
    });
  });

export const type = (stream: stream.Writable, input: string) =>
  new Promise<void>((resolve, reject) => {
    stream.write(input, (err) => {
      if (err != null) {
        return reject(err);
      }
      resolve();
    });
  });

export const createInteractiveCLIHelpers = ({
  stdin,
  stdout,
}: {
  stdin: stream.Writable;
  stdout: stream.Readable;
}) => ({
  enter: async () => enter(stdin),
  type: async (input: string) => type(stdin, input),
  waitFor: async (
    test: (value: string) => boolean,
    {
      normalize = true,
      ...options
    }: { normalize?: boolean; timeout?: number } = {},
  ) =>
    waitForInStream(
      stdout,
      (chunk) => {
        if (
          typeof chunk === "string" ||
          (typeof chunk === "object" &&
            chunk !== null &&
            "toString" in (chunk as any) &&
            typeof (chunk as any).toString === "function")
        ) {
          // Multiple lines are often written to stdout simultaneously, which is an implementation
          // detail that's difficult to rely on. Split on newlines so tests get predicatable input.
          // eslint-disable-next-line @typescript-eslint/no-base-to-string
          for (const line of chunk.toString().split("\n")) {
            if (test(normalize ? stripAnsi(line) : line)) {
              return true;
            }
          }
        }
        return false;
      },
      options,
    ),
});

export const keys = {
  down: "\x1B\x5B\x42",
  enter: "\x0D",
  up: "\x1B\x5B\x41",
};
